@@ -1,6 +1,7 @@
+import datetime
 import logging
+
 from flask import g
-from flask_appbuilder import Base
 from flask_login import current_user
 from sqlalchemy import MetaData, Table
 from sqlalchemy.engine.reflection import Inspector
@@ -9,11 +10,12 @@ from werkzeug.security import generate_password_hash, check_password_hash
 from flask_login import LoginManager
 from flask_openid import OpenID
 
+from flask_appbuilder import Base
 from models import User, Role, PermissionView, Permission, ViewMenu, \
     assoc_permissionview_role
 from views import AuthDBView, AuthOIDView, ResetMyPasswordView, AuthLDAPView, \
     ResetPasswordView, UserDBGeneralView, UserLDAPGeneralView, UserOIDGeneralView, RoleGeneralView, \
-    PermissionViewGeneralView, ViewMenuGeneralView, PermissionGeneralView
+    PermissionViewGeneralView, ViewMenuGeneralView, PermissionGeneralView, UserStatsChartView
 
 
 log = logging.getLogger(__name__)
@@ -51,7 +53,6 @@ class SecurityManager(object):
             else:
                 raise Exception("No AUTH_LDAP_SERVER defined on config with AUTH_LDAP authentication type.")
 
-
         self.lm = LoginManager(app)
         self.lm.login_view = 'login'
         self.oid = OpenID(app)
@@ -82,6 +83,7 @@ class SecurityManager(object):
         role_view = baseapp._init_view_session(RoleGeneralView)
         baseapp.add_view(role_view, "List Roles", "/roles/list", "fa-group", "Security")
         role_view.related_views = [user_view.__class__]
+        baseapp.add_view(baseapp._init_view_session(UserStatsChartView), "User's Statistics", icon="fa-bar-chart-o", category="Security")
         baseapp.menu.add_separator("Security")
         baseapp.add_view(baseapp._init_view_session(PermissionViewGeneralView), "Base Permissions", "/permissions/list",
                          "fa-lock", "Security")
@@ -157,48 +159,69 @@ class SecurityManager(object):
 
 
     def init_db(self):
-        engine = self.session.get_bind(mapper=None, clause=None)
+        try:
+            engine = self.session.get_bind(mapper=None, clause=None)
 
-        inspector = Inspector.from_engine(engine)
-        if 'ab_user' not in inspector.get_table_names():
-            log.info("Security DB not found Creating")
-            Base.metadata.create_all(engine)
-            log.info("Security DB Created")
-            self.migrate_db()
-        if self.session.query(Role).filter_by(name=self.auth_role_admin).first() is None:
-            role = Role()
-            role.name = self.auth_role_admin
-            self.session.add(role)
-            self.session.commit()
-            log.info("Inserted Role for public access %s" % (self.auth_role_admin))
-        if not self.session.query(Role).filter_by(name=self.auth_role_public).first():
-            role = Role()
-            role.name = self.auth_role_public
-            self.session.add(role)
-            self.session.commit()
-            log.info("Inserted Role for public access %s" % (self.auth_role_public))
-        if not self.session.query(User).all():
-            user = User()
-            user.first_name = 'Admin'
-            user.last_name = 'User'
-            user.username = 'admin'
-            user.password = 'general'
-            user.active = True
-            user.role = self.session.query(Role).filter_by(name=self.auth_role_admin).first()
-            self.session.add(user)
-            self.session.commit()
-            log.info("Inserted initial Admin user")
-            log.info("Login using Admin/general")
+            inspector = Inspector.from_engine(engine)
+            if 'ab_user' not in inspector.get_table_names():
+                log.info("Security DB not found Creating")
+                Base.metadata.create_all(engine)
+                log.info("Security DB Created")
+                self.migrate_db()
+            if self.session.query(Role).filter_by(name=self.auth_role_admin).first() is None:
+                role = Role()
+                role.name = self.auth_role_admin
+                self.session.add(role)
+                self.session.commit()
+                log.info("Inserted Role for public access %s" % (self.auth_role_admin))
+            if not self.session.query(Role).filter_by(name=self.auth_role_public).first():
+                role = Role()
+                role.name = self.auth_role_public
+                self.session.add(role)
+                self.session.commit()
+                log.info("Inserted Role for public access %s" % (self.auth_role_public))
+            if not self.session.query(User).all():
+                user = User()
+                user.first_name = 'Admin'
+                user.last_name = 'User'
+                user.username = 'admin'
+                user.password = generate_password_hash('general')
+                user.email = 'admin@fab.org'
+                user.active = True
+                user.role = self.session.query(Role).filter_by(name=self.auth_role_admin).first()
+                self.session.add(user)
+                self.session.commit()
+                log.info("Inserted initial Admin user")
+                log.info("Login using Admin/general")
+        except Exception as e:
+            log.error("DB Creation and initialization failed, if just upgraded to 0.7.X you must migrate the DB. {0}".format(str(e)))
 
 
+    """
+    ----------------------------------------
+        AUTHENTICATION METHODS
+    ----------------------------------------
+    """
     def auth_user_db(self, username, password):
+        """
+            Method for authenticating user, auth db style
+
+            :param username:
+                The username
+            :param password:
+                The password, will be tested against hashed password on db
+        """
         if username is None or username == "":
             return None
-        user = self.session.query(User).filter_by(username=username, password=password).first()
+        user = self.session.query(User).filter_by(username=username).first()
         if user is None or (not user.is_active()):
             return None
-        else:
+        elif check_password_hash(user.password, password):
+            self._update_user_auth_stat(user, True)
             return user
+        else:
+            self._update_user_auth_stat(user, False)
+            return None
 
     def auth_user_ldap(self, username, password):
         if username is None or username == "":
@@ -216,34 +239,72 @@ class SecurityManager(object):
                 con.set_option(ldap.OPT_REFERRALS, 0)
                 try:
                     con.bind_s(username, password)
+                    self._update_user_auth_stat(user)
                     return user
                 except ldap.INVALID_CREDENTIALS:
-                    log.error("INVALID {} {}".format(username, password))
+                    self._update_user_auth_stat(user, False)
                     return None
             except ldap.LDAPError, e:
                 if type(e.message) == dict and e.message.has_key('desc'):
-                    log.error(e.message['desc'])
+                    log.error("LDAP Error {0}".format(e.message['desc']))
                     return None
                 else:
                     log.error(e)
                     return None
 
     def auth_user_oid(self, email):
+        """
+            OpenID user Authentication
+
+            :type self: User model
+        """
         user = self.session.query(User).filter_by(email=email).first()
-        if user is None or (not user.is_active()):
+        if user is None:
+            self._update_user_auth_stat(user, False)
+            return None
+        elif not user.is_active():
             return None
         else:
+            self._update_user_auth_stat(user)
             return user
 
+    def _update_user_auth_stat(self, user, success=True):
+        """
+            Update authentication successful to user.
+
+            :param user:
+                The authenticated user model
+        """
+        try:
+            if not user.login_count:
+                user.login_count = 0
+            elif not user.fail_login_count:
+                user.fail_login_count = 0
+            if success:
+                user.login_count += 1
+                user.fail_login_count = 0
+            else:
+                user.fail_login_count += 1
+            user.last_login = datetime.datetime.now()
+            self.session.merge(user)
+            self.session.commit()
+        except Exception as e:
+            log.error("Update user login stat: {0}".format(str(e)))
+            self.session.rollback()
+
+
     def reset_password(self, userid, password):
-        user = self.get_user_by_id(userid)
-        user.password = password
-        self.session.commit()
+        try:
+            user = self.get_user_by_id(userid)
+            user.password = generate_password_hash(password)
+            self.session.commit()
+        except Exception as e:
+            log.error("Reset password: {0}".format(str(e)))
+            self.session.rollback()
 
     def get_user_by_id(self, pk):
         return self.session.query(User).get(pk)
 
-
     def _get_auth_type(self, app):
         if 'AUTH_TYPE' in app.config:
             return app.config['AUTH_TYPE']
@@ -266,6 +327,12 @@ class SecurityManager(object):
             return 'Public'
 
 
+    """
+        ----------------------------------------
+            PERMISSION ACCESS CHECK
+        ----------------------------------------
+    """
+
     def is_item_public(self, permission_name, view_name):
         """
             Check if view has public permissions
@@ -314,6 +381,14 @@ class SecurityManager(object):
                 return False
         return False
 
+
+
+    """
+        ----------------------------------------
+            PERMISSION MANAGEMENT
+        ----------------------------------------
+    """
+
     def _add_permission(self, name):
         """
             Adds a permission to the backend, model permission
@@ -322,12 +397,16 @@ class SecurityManager(object):
                 name of the permission to add: 'can_add','can_edit' etc...
         """
         perm = self.session.query(Permission).filter_by(name=name).first()
-        if perm == None:
-            perm = Permission()
-            perm.name = name
-            self.session.add(perm)
-            self.session.commit()
-            return perm
+        if perm is None:
+            try:
+                perm = Permission()
+                perm.name = name
+                self.session.add(perm)
+                self.session.commit()
+                return perm
+            except Exception as e:
+                log.error("Add Permission: {0}".format(str(e)))
+                self.session.rollback()
         return perm
 
 
@@ -339,11 +418,15 @@ class SecurityManager(object):
         """
         view_menu = self.session.query(ViewMenu).filter_by(name=view_name).first()
         if view_menu is None:
-            view_menu = ViewMenu()
-            view_menu.name = view_name
-            self.session.add(view_menu)
-            self.session.commit()
-            return view_menu
+            try:
+                view_menu = ViewMenu()
+                view_menu.name = view_name
+                self.session.add(view_menu)
+                self.session.commit()
+                return view_menu
+            except Exception as e:
+                log.error("Add View Menu Error: {0}".format(str(e)))
+                self.session.rollback()
         return view_menu
 
     def _add_permission_view_menu(self, permission_name, view_menu_name):
@@ -359,24 +442,33 @@ class SecurityManager(object):
         perm = self._add_permission(permission_name)
         pv = PermissionView()
         pv.view_menu_id, pv.permission_id = vm.id, perm.id
-        self.session.add(pv)
-        self.session.commit()
-        log.info("Added Permission View %s" % (str(pv)))
-        return pv
+        try:
+            self.session.add(pv)
+            self.session.commit()
+            log.info("Added Permission View %s" % (str(pv)))
+            return pv
+        except Exception as e:
+            log.error("Add Permission to View Error: {0}".format(str(e)))
+            self.session.rollback()
+
 
     def _del_permission_view_menu(self, permission_name, view_menu_name):
-        perm = self.session.query(Permission).filter_by(name=permission_name).first()
-        vm = self.session.query(ViewMenu).filter_by(name=view_menu_name).first()
-        pv = self.session.query(PermissionView).filter_by(permission=perm, view_menu=vm).first()
-        # delete permission on view
-        self.session.delete(pv)
-        self.session.commit()
-        # if last permission delete permission
-        pv = self.session.query(PermissionView).filter_by(permission=perm).all()
-        if not pv:
-            self.session.delete(perm)
+        try:
+            perm = self.session.query(Permission).filter_by(name=permission_name).first()
+            vm = self.session.query(ViewMenu).filter_by(name=view_menu_name).first()
+            pv = self.session.query(PermissionView).filter_by(permission=perm, view_menu=vm).first()
+            # delete permission on view
+            self.session.delete(pv)
             self.session.commit()
-        log.info("Removed Permission View %s" % (str(permission_name)))
+            # if last permission delete permission
+            pv = self.session.query(PermissionView).filter_by(permission=perm).all()
+            if not pv:
+                self.session.delete(perm)
+                self.session.commit()
+            log.info("Removed Permission View %s" % (str(permission_name)))
+        except Exception as e:
+            log.error("Del Permission from View Error: {0}".format(str(e)))
+            self.session.rollback()
 
 
     def _find_permission(self, lst, item):
@@ -385,6 +477,12 @@ class SecurityManager(object):
                 return True
         return False
 
+    def _find_permission_view(self, lst, permission, view_menu):
+        for i in lst:
+            if i.permission.name == permission and i.view_menu.name == view_menu:
+                return True
+        return False
+
     def add_permissions_view(self, base_permissions, view_menu):
         """
             Adds a permission on a view menu to the backend
@@ -397,29 +495,34 @@ class SecurityManager(object):
         view_menu_db = self.session.query(ViewMenu).filter_by(name=view_menu).first()
         if view_menu_db is None:
             view_menu_db = self._add_view_menu(view_menu)
-        lst = self.session.query(PermissionView).filter_by(view_menu_id=view_menu_db.id).all()
+        perm_views = self.session.query(PermissionView).filter_by(view_menu_id=view_menu_db.id).all()
 
-        if not lst:
-            # No permissions for this view
+        if not perm_views:
+            # No permissions yet on this view
             for permission in base_permissions:
                 pv = self._add_permission_view_menu(permission, view_menu)
                 role_admin = self.session.query(Role).filter_by(name=self.auth_role_admin).first()
                 self.add_permission_role(role_admin, pv)
         else:
+            # Permissions on this view exist but....
+            role_admin = self.session.query(Role).filter_by(name=self.auth_role_admin).first()
             for permission in base_permissions:
-                if not self._find_permission(lst, permission):
+                # Check if base view permissions exist
+                if not self._find_permission(perm_views, permission):
                     pv = self._add_permission_view_menu(permission, view_menu)
-                    role_admin = self.session.query(Role).filter_by(name=self.auth_role_admin).first()
                     self.add_permission_role(role_admin, pv)
-            for item in lst:
-                if item.permission.name not in base_permissions:
+            for perm_view in perm_views:
+                if perm_view.permission.name not in base_permissions:
                     # perm to delete
                     roles = self.session.query(Role).all()
-                    pv = self.session.query(Permission).filter_by(name=item.permission.name).first()
+                    perm = self.session.query(Permission).filter_by(name=perm_view.permission.name).first()
                     # del permission from all roles
                     for role in roles:
-                        self.del_permission_role(role, pv)
-                    self._del_permission_view_menu(item.permission.name, view_menu)
+                        self.del_permission_role(role, perm)
+                    self._del_permission_view_menu(perm_view.permission.name, view_menu)
+                elif perm_view not in role_admin.permissions:
+                    # Role Admin must have all permissions
+                    self.add_permission_role(role_admin, perm_view)
 
 
     def add_permissions_menu(self, view_menu_name):
@@ -448,10 +551,15 @@ class SecurityManager(object):
                 The PermissionViewMenu object
         """
         if perm_view not in role.permissions:
-            role.permissions.append(perm_view)
-            self.session.merge(role)
-            self.session.commit()
-            log.info("Added Permission %s to role %s" % (str(perm_view), role.name))
+            try:
+                role.permissions.append(perm_view)
+                self.session.merge(role)
+                self.session.commit()
+                log.info("Added Permission %s to role %s" % (str(perm_view), role.name))
+            except Exception as e:
+                log.error("Add Permission to Role Error: {0}".format(str(e)))
+                self.session.rollback()
+
 
     def del_permission_role(self, role, perm_view):
         """
@@ -463,7 +571,11 @@ class SecurityManager(object):
                 The PermissionViewMenu object
         """
         if perm_view in role.permissions:
-            role.permissions.remove(perm_view)
-            self.session.merge(role)
-            self.session.commit()
-            log.info("Removed Permission %s to role %s" % (str(perm_view), role.name))
+            try:
+                role.permissions.remove(perm_view)
+                self.session.merge(role)
+                self.session.commit()
+                log.info("Removed Permission %s to role %s" % (str(perm_view), role.name))
+            except Exception as e:
+                log.error("Del Permission to Role Error: {0}".format(str(e)))
+                self.session.rollback()
