@@ -1,10 +1,12 @@
+import datetime
 from ..fieldwidgets import BS3PasswordFieldWidget
 from flask_wtf import validators
 from flask import render_template, flash, redirect, session, url_for, request, g
-from openid.consumer import discover
+from werkzeug.security import generate_password_hash
+#from openid.consumer import discover
 from openid.consumer.consumer import Consumer, SUCCESS, CANCEL
-from openid.extensions import ax
-from openid.extensions.sreg import SRegRequest, SRegResponse
+#from openid.extensions import ax
+#from openid.extensions.sreg import SRegRequest, SRegResponse
 from flask.ext.openid import SessionWrapper, OpenIDResponse
 from flask_wtf import EqualTo, PasswordField
 from flask.ext.babelpkg import gettext, lazy_gettext
@@ -12,7 +14,9 @@ from flask_login import login_user, logout_user
 
 from flask_appbuilder.models.datamodel import SQLAModel
 from flask_appbuilder.views import BaseView, GeneralView, SimpleFormView, expose
+from flask_appbuilder.charts.views import DirectChartView
 from ..actions import action
+from .._compat import as_unicode
 from forms import LoginForm_db, LoginForm_oid, ResetPasswordForm
 from models import User, Permission, PermissionView, Role, ViewMenu
 from decorators import has_access
@@ -74,7 +78,7 @@ class ResetMyPasswordView(SimpleFormView):
 
     def form_post(self, form):
         self.baseapp.sm.reset_password(g.user.id, form.password.data)
-        flash(unicode(self.message), 'info')
+        flash(as_unicode(self.message), 'info')
 
 
 class ResetPasswordView(SimpleFormView):
@@ -93,7 +97,7 @@ class ResetPasswordView(SimpleFormView):
     def form_post(self, form):
         pk = request.args.get('pk')
         self.baseapp.sm.reset_password(pk, form.password.data)
-        flash(unicode(self.message), 'info')
+        flash(as_unicode(self.message), 'info')
 
 
 class UserGeneralView(GeneralView):
@@ -112,7 +116,15 @@ class UserGeneralView(GeneralView):
                      'password': lazy_gettext('Password'),
                      'active': lazy_gettext('Is Active?'),
                      'email': lazy_gettext('EMail'),
-                     'role': lazy_gettext('Role')}
+                     'role': lazy_gettext('Role'),
+                     'last_login': lazy_gettext('Last login'),
+                     'login_count': lazy_gettext('Login count'),
+                     'fail_login_count': lazy_gettext('Failed login count'),
+                     'created_on': lazy_gettext('Created on'),
+                     'created_by': lazy_gettext('Created by'),
+                     'changed_on': lazy_gettext('Changed on'),
+                     'changed_by': lazy_gettext('Changed by')}
+
     description_columns = {'first_name': lazy_gettext('Write the user first name or names'),
                            'last_name': lazy_gettext('Write the user last name'),
                            'username': lazy_gettext(
@@ -124,18 +136,35 @@ class UserGeneralView(GeneralView):
                            'role': lazy_gettext(
                                'The user role on the application, this will associate with a list of permissions'),
                            'conf_password': lazy_gettext('Please rewrite the users password to confirm')}
-    list_columns = ['full_name', 'username', 'email', 'active', 'role']
-    show_columns = ['first_name', 'last_name', 'username', 'active', 'email', 'role']
-    order_columns = ['username', 'email']
-    search_columns = ['first_name', 'last_name', 'username', 'email', 'role']
+
+    list_columns = ['first_name', 'last_name', 'username', 'email', 'active', 'role']
+
+    show_fieldsets = [
+        ('User info',
+            {'fields': ['username', 'active', 'role', 'login_count']}),
+        ('Personal Info',
+            {'fields': ['first_name', 'last_name', 'email'], 'expanded': True}),
+        ('Audit Info',
+            {'fields': ['last_login', 'fail_login_count', 'created_on',
+                        'created_by', 'changed_on', 'changed_by'], 'expanded': False}),
+    ]
+
+    user_show_fieldsets = [
+        ('User info',
+            {'fields': ['username', 'active', 'role', 'login_count']}),
+        ('Personal Info',
+            {'fields': ['first_name', 'last_name', 'email'], 'expanded': True}),
+    ]
+
+    order_columns = ['first_name', 'last_name', 'username', 'email']
+    search_columns = ['first_name', 'last_name', 'username', 'email', 'role', 
+                    'created_by', 'changed_by', 'changed_on','changed_by', 'login_count']
 
     add_columns = ['first_name', 'last_name', 'username', 'active', 'email', 'role']
     edit_columns = ['first_name', 'last_name', 'username', 'active', 'email', 'role']
 
     user_info_title = lazy_gettext("Your user information")
 
-    show_additional_links = []
-
 
 class UserOIDGeneralView(UserGeneralView):
     @expose('/userinfo/')
@@ -195,7 +224,7 @@ class UserDBGeneralView(UserGeneralView):
     def userinfo(self):
         actions = {}
         actions['resetmypassword'] = self.actions.get('resetmypassword')
-        widgets = self._get_show_widget(g.user.id, actions=actions)
+        widgets = self._get_show_widget(g.user.id, actions=actions, show_fieldsets=self.user_show_fieldsets)
         return render_template(self.show_template,
                                title=self.user_info_title,
                                widgets=widgets,
@@ -211,6 +240,24 @@ class UserDBGeneralView(UserGeneralView):
         return redirect(url_for('ResetPasswordView.this_form_get', pk=item.id))
 
 
+    def pre_update(self, item):
+        item.changed_on = datetime.datetime.now()
+        item.changed_by_fk = g.user.id
+        #item.password = generate_password_hash(item.password)
+
+    def pre_add(self, item):
+        item.password = generate_password_hash(item.password)
+
+
+class UserStatsChartView(DirectChartView):
+    datamodel = SQLAModel(User)
+    chart_title = lazy_gettext('User Statistics')
+    label_columns = UserGeneralView.label_columns
+    search_columns = UserGeneralView.search_columns
+    direct_columns = {'Login Count': ('username', 'login_count'),
+                      'Failed Login Count': ('username', 'fail_login_count')}
+
+
 class RoleGeneralView(GeneralView):
     route_base = '/roles'
 
@@ -257,7 +304,7 @@ class AuthDBView(AuthView):
         if form.validate_on_submit():
             user = self.baseapp.sm.auth_user_db(form.username.data, form.password.data)
             if not user:
-                flash(unicode(self.invalid_login_message), 'warning')
+                flash(as_unicode(self.invalid_login_message), 'warning')
                 return redirect('/login')
             login_user(user, remember=False)
             return redirect('/')
@@ -279,7 +326,7 @@ class AuthLDAPView(AuthView):
         if form.validate_on_submit():
             user = self.baseapp.sm.auth_user_ldap(form.username.data, form.password.data)
             if not user:
-                flash(unicode(self.invalid_login_message), 'warning')
+                flash(as_unicode(self.invalid_login_message), 'warning')
                 return redirect('/login')
             login_user(user, remember=False)
             return redirect('/')
@@ -325,11 +372,11 @@ class AuthOIDView(AuthView):
 
     def after_login(self, resp):
         if resp.email is None or resp.email == "":
-            flash(unicode(self.invalid_login_message), 'warning')
+            flash(as_unicode(self.invalid_login_message), 'warning')
             return redirect('appbuilder/general/security/login_oid.html')
         user = self.baseapp.sm.auth_user_oid(resp.email)
         if user is None:
-            flash(unicode(self.invalid_login_message), 'warning')
+            flash(as_unicode(self.invalid_login_message), 'warning')
             return redirect('appbuilder/general/security/login_oid.html')
         remember_me = False
         if 'remember_me' in session:
