@@ -1,10 +1,11 @@
 import logging
-from flask import render_template, flash, redirect, url_for, request, send_file
+from flask import render_template, flash, redirect, send_file
 from .filemanager import uuid_originalname
 from .security.decorators import has_access
-from .widgets import FormWidget, GroupFormListWidget
-from .actions import action
+from .widgets import FormWidget, GroupFormListWidget, ListMasterWidget
 from .baseviews import expose, BaseView, BaseCRUDView
+from urltools import *
+
 
 log = logging.getLogger(__name__)
 
@@ -228,6 +229,55 @@ class GeneralView(BaseCRUDView):
             return redirect('.')
 
 
+class MasterDetailView(BaseCRUDView):
+    """
+        Implements behaviour for controlling two CRUD views
+        linked by PK and FK, in a master/detail type with
+        two lists.
+
+        Master view will behave like a left menu::
+
+            class DetailView(GeneralView):
+                datamodel = SQLAModel(DetailTable, db.session)
+
+            class MasterView(MasterDetailView):
+                datamodel = SQLAModel(MasterTable, db.session)
+                related_views = [DetailView]
+
+    """
+
+    list_template = 'appbuilder/general/model/left_master_detail.html'
+    list_widget = ListMasterWidget
+    master_div_width = 2
+    """
+        Set to configure bootstrap class for master grid size
+    """
+
+    @expose('/list/')
+    @expose('/list/<pk>')
+    @has_access
+    def list(self, pk=None):
+        pages = get_page_args()
+        page_sizes = get_page_size_args()
+        orders = get_order_args()
+
+        widgets = self._list()
+        if pk:
+            item = self.datamodel.get(pk)
+            widgets = self._get_related_list_widgets(item, orders=orders,
+                                                     pages=pages, page_sizes=page_sizes, widgets=widgets)
+            related_views = self._related_views
+        else:
+            related_views = []
+
+        return render_template(self.list_template,
+                               title=self.list_title,
+                               widgets=widgets,
+                               related_views=related_views,
+                               master_div_width = self.master_div_width,
+                               baseapp=self.baseapp)
+
+
 class CompactCRUDMixin(BaseCRUDView):
     """
         Mix with GeneralView to implement a list with add and edit on the same page.
@@ -239,12 +289,11 @@ class CompactCRUDMixin(BaseCRUDView):
     def _get_list_widget(self, **args):
         """ get joined base filter and current active filter for query """
         widgets = super(CompactCRUDMixin, self)._get_list_widget(**args)
-        ret_widget = {}
-        ret_widget['list'] = GroupFormListWidget(list_widget=widgets.get('list'),
-                                                 form_widget=self._session_form_widget,
-                                                 form_action=self._session_form_action,
-                                                 form_title=self._session_form_title)
-        return ret_widget
+        return {'list': GroupFormListWidget(list_widget=widgets.get('list'),
+                                            form_widget=self._session_form_widget,
+                                            form_action=self._session_form_action,
+                                            form_title=self._session_form_title)}
+
 
     @expose('/list/', methods=['GET', 'POST'])
     @has_access
